program DpGLApp;

uses Windows,Messages,GL,GLU,ASE;//,Dialogs,SysUtils;

const AppName    = 'DpGLApp';
      AppCaption = 'Delphi OpenGL Application Frame';

var hWindow:HWnd;
    Window_Width :integer = 500;
    Window_Height:integer = 500;

    FDC:HDC;
    FRC:HGLRC;
    FPalette:HPALETTE;
    FlyModel:TASEFile;
    zoom:Integer = 15;
    ax  :Integer = 0;
    ay  :Integer = 0;
    az  :Integer = 0;
    m_Point:TPoint;
    model_id:integer = 1;
    models : array[1..6] of string = ('AH64','A10','MIG21','F16','F117A','F22');

    World_TimeID:Cardinal;


//==============OpenGL Programing Block =============================
procedure Init_MainForm_DC(DC: HDC; Palette: HPalette);
var hHeap: Integer;
    nColors,i:Integer;
    lpPalette:PLogPalette;
    byRedMask,byGreenMask,byBlueMask:Byte;
    nPixelFormat:Integer;
    pfd:TPixelFormatDescriptor;
begin
  // Initialise the form's DC for OpenGL, and setup a palette.
  FillChar(pfd, SizeOf(pfd), 0);
  with pfd do
  begin
    nSize        := SizeOf(TPixelFormatDescriptor);           // Size of this structure
    nVersion     := 1;                                        // Version number
    dwFlags      := PFD_DRAW_TO_WINDOW or
                    PFD_SUPPORT_OPENGL;                       // Flags
    dwFlags      := dwFlags or PFD_DOUBLEBUFFER;              // Double buffering = TRUE
    iPixelType   := PFD_TYPE_RGBA;                            // RGBA pixel values
    cColorBits   := 24;                                       // 24-bit color
    cDepthBits   := 24;                                       // 24-bit depth buffer
    cStencilBits := 8;                                        // Stencil buffer, too.
    iLayerType   := PFD_MAIN_PLANE;                           // Layer type
  end;
  //------------------------------------------------------------------------------------
  nPixelFormat := ChoosePixelFormat(DC, @pfd);
  SetPixelFormat(DC, nPixelFormat, @pfd);
  //------------------------------------------------------------------------------------
  DescribePixelFormat(DC, nPixelFormat, SizeOf(TPixelFormatDescriptor), pfd);
  //------------------------------------------------------------------------------------
  if((pfd.dwFlags and PFD_NEED_PALETTE) <> 0)then
    begin
      nColors   := 1 shl pfd.cColorBits;
      hHeap     := GetProcessHeap;
      lpPalette := HeapAlloc(hHeap, 0, sizeof(TLogPalette) + (nColors * sizeof(TPaletteEntry)));
      //-----------------------------------------------------------------------------------------
      lpPalette^.palVersion := $300;
      lpPalette^.palNumEntries := nColors;
      //----------------------------------------
      byRedMask   := (1 shl pfd.cRedBits)   - 1;
      byGreenMask := (1 shl pfd.cGreenBits) - 1;
      byBlueMask  := (1 shl pfd.cBlueBits)  - 1;
      //----------------------------------------
      for i := 0 to nColors - 1 do
        begin
          lpPalette^.palPalEntry[i].peRed
            := (((i shr pfd.cRedShift)   and byRedMask)   * 255) div byRedMask;
          lpPalette^.palPalEntry[i].peGreen
            := (((i shr pfd.cGreenShift) and byGreenMask) * 255) div byGreenMask;
          lpPalette^.palPalEntry[i].peBlue
            := (((i shr pfd.cBlueShift)  and byBlueMask)  * 255) div byBlueMask;
          lpPalette^.palPalEntry[i].peFlags := 0;
        end;//for...i...end!!
      //----------------------------------------
      Palette := CreatePalette(lpPalette^);
      HeapFree(hHeap, 0, lpPalette);
      //----------------------------------------
      if Palette <> 0 then
        begin
          SelectPalette(DC, Palette, FALSE);
          RealizePalette(DC);
        end;//if...end!!
    end;//if...end!!
end;

procedure PageFlip;
begin
  // Do a page flip.
  SwapBuffers(FDC);
end;

procedure InitOpenGL;
const
  //=====================OpenGL Lights============================
  ambientLight : array [0..3] of GLfloat = (3.0,3.0,3.0,1);
  diffuseLight : array [0..3] of GLfloat = (0.5,0.5,0.5,1);
  specularLight: array [0..3] of GLfloat = (0.3,0.3,0.3,1);
begin
  //Initialise OpenGL. Get a DC and create an RC.
  FDC := GetDC(hWindow);
  Init_MainForm_DC(FDC, FPalette);
  FRC := wglCreateContext(FDC);
  //Make this form the active OpenGL rendering context.
  if((FDC <> 0)and(FRC <> 0))then wglMakeCurrent(FDC, FRC);
  //=======================================================
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_CULL_FACE);
  glEnable(GL_LIGHTING);

  glLightfv(GL_LIGHT0,GL_AMBIENT ,@ambientLight);
  glLightfv(GL_LIGHT0,GL_DIFFUSE ,@diffuseLight);
  glLightfv(GL_LIGHT0,GL_SPECULAR,@specularLight);
  glEnable(GL_LIGHT0);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  gluPerspective(60, Window_Width/Window_Height, 1, 2000);
  glMatrixMode(GL_MODELVIEW);
  glViewport(0, 0, Window_Width,Window_Height);
end;

procedure Release_OpenGL;
begin
  // Clean up OpenGL resources. Called automatically by form destructor.
  if(FRC <> 0)then
    begin
      wglMakeCurrent(0, 0);
      wglDeleteContext(FRC);
    end;//if...end!!
  if(FPalette <> 0)then DeleteObject(FPalette);
  ReleaseDC(hWindow, FDC);
end;

procedure RenderMesh(Model3D: TASEMesh;AMBIENT,DIFFUSE,SPECULAR:TASEColor;Shininess:single);
var f:integer;
begin
  //TODO: This doesn't take smoothing groups into account.
  glEnable(GL_LIGHTING);
  glEnable(GL_COLOR_MATERIAL);
  //-------------------------------------------------------
  glColorMaterial(GL_FRONT,GL_AMBIENT);
  glColor4f(AMBIENT.R,AMBIENT.G,AMBIENT.B,AMBIENT.A);
  //-------------------------------------------------------
  glColorMaterial(GL_FRONT,GL_DIFFUSE);
  glColor4f(DIFFUSE.R,DIFFUSE.G,DIFFUSE.B,DIFFUSE.A);
  //-------------------------------------------------------
  glColorMaterial(GL_FRONT,GL_SPECULAR);
  glColor4f(SPECULAR.R,SPECULAR.G,SPECULAR.B,SPECULAR.A);
  //-------------------------------------------------------
  glMaterialf(GL_FRONT,GL_SHININESS,Shininess);
  //=======================================================
  glBegin(GL_TRIANGLES);
    for f := 0 to Model3D.FaceList.NumFaces - 1 do
      begin
        if(Model3D.Normals <> nil)and(Model3D.Normals.NumFaceNormals > 0) then
          glNormal3f( Model3D.Normals.FaceNormals[f].X
                     ,Model3D.Normals.FaceNormals[f].Y
                     ,Model3D.Normals.FaceNormals[f].Z);

        glVertex3f( Model3D.VertexList.Vertices[Model3D.FaceList.Faces[f].A].X
                   ,Model3D.VertexList.Vertices[Model3D.FaceList.Faces[f].A].Y
                   ,Model3D.VertexList.Vertices[Model3D.FaceList.Faces[f].A].Z);

        glVertex3f( Model3D.VertexList.Vertices[Model3D.FaceList.Faces[f].B].X
                   ,Model3D.VertexList.Vertices[Model3D.FaceList.Faces[f].B].Y
                   ,Model3D.VertexList.Vertices[Model3D.FaceList.Faces[f].B].Z);

        glVertex3f( Model3D.VertexList.Vertices[Model3D.FaceList.Faces[f].C].X
                   ,Model3D.VertexList.Vertices[Model3D.FaceList.Faces[f].C].Y
                   ,Model3D.VertexList.Vertices[Model3D.FaceList.Faces[f].C].Z);
      end;//for...f...end!!
  glEnd;
end;

Procedure DrawGroundand;
begin
  //a
  //glBindTexture(GL_TEXTURE_2D,SkyTex[4]);
  glColor3f(1,2,1);
  glBegin(GL_QUADS);
    glTexCoord2f(20,20);glVertex3f(-563,-3,-563);
    glTexCoord2f(20, 30);glVertex3f(-563,-3, 563);
    glTexCoord2f( 30, 30);glVertex3f( 563,-3, 563);
    glTexCoord2f( 30,20);glVertex3f( 563,-3,-563);
  glEnd;
end;

procedure Render_3D_Scene;
var i,j,MaterialID:Integer;
    mat:TASEMaterial;
begin
  glClearColor(0.5,0.7,0.8, 0);
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
  glLoadIdentity;
  glTranslatef(0, 0, -zoom);
  glRotatef(-90, 1, 0, 0);
  //----------------------------------------------------
  glRotatef(ay, 0, 0, 1);
  //----------------------------------------------------
  if(FlyModel <> nil)then
  for i := 0 to FlyModel.NumGeomObjects - 1 do
    begin
      mat := nil;
      MaterialID := FlyModel.GeomObjects[i].MaterialRef;
      mat := FlyModel.MaterialList.Materials[MaterialID];
      if(mat <> nil)then
        with FlyModel.GeomObjects[i] do
          RenderMesh(Mesh,mat.Ambient
                         ,mat.Diffuse
                         ,mat.Specular,mat.Shine);
    end;
  mat := nil;
  //glRotatef(90, 1, 0, 0);
  (*
  //=====================================================
  for j := 0 to FlyModel.NumGroups - 1 do
    begin
      for i := 0 to FlyModel.Groups[j].NumGeomObjects - 1 do
        begin
          mat := nil;
          MaterialID := FlyModel.GeomObjects[i].MaterialRef;
          mat := FlyModel.MaterialList.Materials[MaterialID];
          if(mat <> nil)then
            with FlyModel.GeomObjects[i] do
              RenderMesh(Mesh,mat.Ambient
                             ,mat.Diffuse
                             ,mat.Specular,mat.Shine);
        end;
    end;
  //=================================================================
  *)

  //=================================================================
  PageFlip;
end;

//========================================================================================
procedure World_Time_Goes_On_Proc(Window:HWND;message,idEvent:UINT;dwTime:DWORD); stdcall;
begin
  ay := ay+1;
  Render_3D_Scene;
end;  

//========================================================================================
// WindowProc Callback
function WindowProc(Window: HWnd; AMessage, wParam, lParam: longint): longint;
  stdcall; export;
begin
  WindowProc := 0;
  case AMessage of
    wm_Destroy:
      begin
        if(FlyModel <> nil)then FlyModel.Free;
        Release_OpenGL;
        PostQuitMessage(0);
        Sleep(10);
        Exit;
      end;
     wm_Size:
       begin
         glMatrixMode(GL_PROJECTION);
         glLoadIdentity;
         gluPerspective(60, LOWORD(lParam)/HIWORD(lParam), 1, 2000);
         glMatrixMode(GL_MODELVIEW);
         glViewport(0, 0, LOWORD(lParam),HIWORD(lParam));
       end;
     WM_MOUSEMOVE:
       begin
         if(MK_LBUTTON = wParam)then
           begin
             ax := ax + (HIWORD(lParam) - m_Point.Y);
             ay := ay + (LOWORD(lParam) - m_Point.X);
           end//if...end!!
         else
           if(MK_RBUTTON = wParam)then
             begin
               zoom := zoom - (HIWORD(lParam) - m_Point.y);
               if(zoom < 1)then zoom := 1
               else
                 if zoom > 1000 then zoom := 1000;
             end;//else...if...end!!
         m_Point.X := LOWORD(lParam);
         m_Point.Y := HIWORD(lParam);    
       end;
     WM_KEYDOWN:
       begin
         ///debug !! ShowMessage(IntToStr(wParam));
         case wParam of
           38 ://UP
              begin
                ax := ax-1;
              end;
           40 ://DOWN
              begin
                ax := ax+1;
              end;
           37 ://LEFT
              begin
                ay := ay-1;
              end;
           39 ://RIGHT
              begin
                ay := ay+1;
              end;
           27 ://ESC
              begin
                PostQuitMessage(0);
                Exit;
              end;
           219://'['
              begin
                if((model_id-1) >= 1)then model_id := model_id-1
                else model_id := 6;
                FlyModel.Free;
                FlyModel := TAseFile.Create;
                FlyModel.LoadFromFile(models[model_id]+'.ASE');
              end;
           221://']'
              begin
                if((model_id+1) <= 6)then model_id := model_id+1
                else model_id := 1;
                FlyModel.Free;
                FlyModel := TAseFile.Create;
                FlyModel.LoadFromFile(models[model_id]+'.ASE');
              end;
         end;//case...end!!
       end;
  end;//case...end!!
  WindowProc := DefWindowProc(Window, AMessage, wParam, lParam);
end;

// register this window to Windows 95
function WinRegister: boolean;
var WindowClass: TWndClass;
begin
  // 1. Fill out the WindowClass record
  WindowClass.Style         := cs_hRedraw or cs_vRedraw;
  WindowClass.lpfnWndProc   := @WindowProc;
  WindowClass.cbClsExtra    := 0;
  WindowClass.cbWndExtra    := 0;
  WindowClass.hInstance     := HInstance;
  WindowClass.hIcon         := LoadIcon(0, idi_Application);
  WindowClass.hCursor       := LoadCursor(0, idc_Arrow);
  WindowClass.hbrBackground := HBrush(Color_Window);
  WindowClass.lpszMenuName  := nil;
  WindowClass.lpszClassName := AppName;
  // 2. pass it to RegisterClass
  Result := RegisterClass(WindowClass) <> 0;
end;

// create and show this window
function WinCreate: HWnd;
var hWindow:HWnd;
begin
  hWindow := CreateWindow( AppName                // address of registered class name
                          ,AppCaption             // address of window name
                          ,ws_OverlappedWindow    // window style
                          ,cw_UseDefault          // horizontal position of window
                          ,cw_UseDefault          // vertical position of window
                          ,Window_Width           // window width
                          ,Window_Height          // window height
                          ,0                      // handle of parent or ower window
                          ,0                      // handle of menu
                          ,HInstance              // handle of application
                          ,nil);                  // address of window-creation data
  if(hWindow <> 0)then
    begin
      ShowWindow(hWindow, cmdShow);
      UpdateWindow(hWindow);
    end;//if...end!!
  Result := hWindow;
end;

//======================= Main Program Block ========================
var AMessage:TMsg;
begin
  // 1. Register the window
  if not WinRegister then
    begin
      MessageBox(0, 'Register failed', nil, mb_Ok);
      Exit;
    end;//if...end!!
  // 2. Create the window
  hWindow := WinCreate;
  if(hWindow = 0)then
    begin
      MessageBox(0, 'WinCreate failed', nil, mb_Ok);
      Exit;
    end//if...end!!
  else
    begin//After Create Window Success,we Initialize the OpenGL
      //Starting Init OpenGL
      InitOpenGL;
      //--------------------------------------------
      //Initialization the model Loder
      FlyModel := TASEFile.Create;
      FlyModel.LoadFromFile(models[model_id]+'.ASE');
      //Get the mouse cursor position
      GetCursorPos(m_Point);
      //======================================================
      World_TimeID := SetTimer(0,0,1,@World_Time_Goes_On_Proc);
    end;//else...end!!
  // 3. Enter the message loop
  while GetMessage(AMessage, 0, 0, 0) do
    begin
      TranslateMessage(AMessage);
      DispatchMessage(AMessage);
    end;
  //4.Enable the 3d world timer!!
  KillTimer(0,World_TimeID);
  //5.End program
  Halt(AMessage.wParam);
end.
